#include <pic.h>				// pic specific identifiers
#define _XTAL_FREQ  4000000		// Xtal speed
__CONFIG(0x3D18);				// Config bits

#define LED_PORT	PORTB		// LED port
#define LED_TRIS	TRISB
#define RS232_PORT	PORTA		// RS232 port configuration
#define RS232_TRIS	TRISA
#define RX			RA7
#define TX			RA6
#define BAUD		84			// 1/9600 = 104uS But! with loop and shift @ 20uS
#define START_BIT	BAUD/2		// Should be 52uS really, but close enough
#define addr		0x78

		// Required prototypes.. each function needs to be declared
		// if called BEFORE definition.

void Serinit(void), SerStringR(char* str), delayMs(int x);
void Serout( unsigned char ch), mtx(unsigned char ch);
int mrx(void), decoder(unsigned char ch);
unsigned char Serin(void);

unsigned char outbuff[10];
unsigned char text[] = {"Nigel "};

void main(void)						// program entry
	{
	int index = 0;
	int ch;							// new buffer variable
	CMCON = 0x7;					// Comparitors off
	LED_TRIS = 0b00000000;			// Led port as outputs
	__delay_ms(150);				// let LCD stabilise
	Serinit();
	LED_PORT = 0;
	while(1)						// endless Loop
		{
		ch = mrx();					// go get a packet
		LED_PORT = ch;				// put byte on portb leds
		}
	}

// I know there are several ways to send and receive using manchester coding
// Using Nigel's original code looks for bit changes, it's probably more efficient 
// But as more and more RF devices now have RS232 I found using this variation
// works for me.. It's mainly to stabilise the PLL anyway....

void mtx(unsigned char ch)
	{
	char x;
	unsigned char sum=0;			// 16 combinations of a nibble
	const unsigned char encode[] = {0xAA,0x9A,0x6A,0x5A,0xA9,0x99,0x69,0x59,
									0xA6,0x96,0x66,0x56,0xA5,0x95,0x65,0x55};

	outbuff[0] = outbuff[1] = 0xAA;			// two preamble bytes
	outbuff[2] = encode[(addr % 16) & 0xf];	// address low
	outbuff[3] = encode[(addr / 16) & 0xf];	// adress high
	outbuff[4] = encode[(ch % 16) & 0xf];	// transmit byte low
	outbuff[5] = encode[(ch / 16) & 0xf];	// transmit byte high
	for(x = 2; x<6; x++)
		sum ^= outbuff[x];					// give a crc to address and data
	outbuff[6] = encode[(sum % 16) & 0xf];	// send crc low byte
	outbuff[7] = encode[(sum / 16) & 0xf];	// send crc high byte
	outbuff[8] = 0;							// null terminated
	SerStringR(outbuff);					// for send routine.
	}

// you could use one more byte (0x55) before the address to signify the start of 
// the data. It will make it a little more stable.

int mrx()
	{
	unsigned char x,ad,crc,sum = 0;

	if(Serin() == 0xAA)					// first preamble
		{
		if(Serin() == 0xAA)				// second preamble
			{
			for(x = 2;x<7;x++)
				{
				outbuff[x] = Serin();							// get 6 bytes
				}
			}
		for(x = 2;x<6;x++)
			sum ^= outbuff[x];									// sum up crc
		crc = decoder(outbuff[7]) * 16 + decoder(outbuff[6]);	// crc compare
		if(sum != crc)
			return 0;
		ad = decoder(outbuff[3]) * 16 + decoder(outbuff[2]);	// big endian
		if(ad != addr)
			return 0;
		x =	decoder(outbuff[5]) * 16 + decoder(outbuff[4]);		// retreive byte
		return x;
		}
	return 0;
	}

int decoder(unsigned char ch)
	{
	unsigned char decode[] = {0xA,0x9,0x6,0x5};		// only 4 combinations
	unsigned char nibL,nibH,x;
	x = ch;
	for(x=0;x<4;x++)
		{
		if((ch & 0x0F) == decode[x])				// loop through the pattern
			nibH = x;								// match?
		if(((ch>>4) & 0xF) == decode[x])			// low byte next (big endian)
			nibL = x;
		}
	x = nibH*4 + nibL;								// retreive nibble value
	return x;
	}

void SerStringR(char* str)		// call serout multiple times
	{
	while(*str != 0)				// until end of string
		Serout(*str++);				// If you use a CODE pointer
	}								// change "char*" to "const char*"

void Serinit()
	{
	RS232_TRIS = 0b10000000;			// RS232  port
	TX = 1;								// initial tx value
	}

unsigned char Serin()					// wait and get a byte on the serial
	{
	char index, ch=0;
	while(1)
		{
		while(RX);						// wait for start bit
		__delay_us(START_BIT);			// Half bit delay
		if(!RX)							// still start?
			{
			__delay_us(BAUD);				// bit delay
			for(index = 0;index<7;index++)
				{
				if(RX)						// bit or no bit
					ch+=0x80;				// add MSB
				__delay_us(BAUD);			// bit delay
				ch>>=1;						// shift along
				}							// until done
			if(RX) ch+=0x80;				// last bit
			return ch;						// return character
			}								// not bothered about the stop bit
		}
	return 0;
	}

void Serout( unsigned char ch)
	{
	char index;
	TX = 0;									// send a start bit
	__delay_us(BAUD);
	for(index = 0; index<8 ; index++)		// send the rest
		{
		if(ch & 0x1)						// test bit
			TX = 1;							// send high
		else
			TX = 0;							// send low
		__delay_us(BAUD);					// bit delay
		ch >>= 1;
		}
	TX = 1;									// send a stop bit
	__delay_us(BAUD);
	}

void delayMs(int x)
	{
	while(x--)
		__delay_ms(1);
	
	}